% an illustrative example of AnchorFree
% please see the algorithm in
% K. Huang, X. Fu, and N. D. Sidiropoulos, ``Anchor-free correlated topic
% modeling: identifiability and algorithm'', NIPS 2016


% cvx is needed to run the demo; download cvx at http://cvxr.com/cvx/

clear;
clc;

% == load the data
load('TDT2.mat')  % matlab version of TDT2 can be found at http://www.cad.zju.edu.cn/home/dengcai/Data/TextData.html
load('9.mat') 

% == pre-processing
fea = tfidf(fea,1); % the tfidf function can be found at http://www.cad.zju.edu.cn/home/dengcai/Data/TextData.html

% - the NCW procedure; see D. Cai, X. He, and J. Han. Locally consistent concept factorization for document clustering. IEEE Trans.
% Knowl. Data Eng., 23(6):902913, 2011.

fea(:,zeroIdx) = [];  % remove the zero rows
fea = fea(sampleIdx,:); 
gnd = gnd(sampleIdx,:); 
D_half = sum(fea*fea',2).^-0.5;
doc_num = size(D_half, 1);
D_half = spdiags(D_half, 0, doc_num, doc_num);
fea = D_half * fea;

% - NCW ends here

%== compute the correlation
D = fea';
[nWords,nDoc]=size(D);
P = (1/nDoc)*D*D';

%== load('data_TDT_F25.mat')
Term = importdata('terms_TDT.txt');
Term.textdata(zeroIdx)=[];
Termwords = Term.textdata;
[nWords,~]=size(P);

F = 5; % number of topics


% == run the algorithm
[V,DD] = eigs(P,F);
B = V*sqrt(abs(DD));
E = eye(F);
M = AnchorFree(B,E);
C = B*M;


%================================================
% display the results
%================================================
K = 20; % display K leading words
for f=1:F
    disp(['--- topic ',num2str(f),':'])
    [~,sort_f] = sort(C(:,f),'descend');
    topic_words_index(f,:)=sort_f(1:K);
    for k=1:K
        topic_words{f,k} = Termwords{ topic_words_index(f,k)};
    end
    for k=1:K
    disp(topic_words{f,k})
    end
    disp('==============================')
end
 


